#!/usr/bin/env python

# Copyright 2007-2009 ITConnection.ru (Digium Distribution, Asterisk services,
# Russia and the C.I.S.)

# support@itconnection.ru

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import ConfigParser
from socket import socket, AF_INET, SOCK_STREAM
import sys, os, re, time, signal

# path to configuration file
#CONFIGFILE = "asx-stats.ini"
CONFIGFILE = "/usr/share/cacti/site/scripts/asx-stats.ini"

KEYS = ["sip.channels", \
	"iax2.channels", \
	"sip.peers.online", \
	"sip.peers.total", \
	"iax2.peers.online", \
	"iax2.peers.total", \
	"sip.users", \
	"iax2.users", \
	"group.channels", \
	"agents.loggedin", \
	"agents.loggedout", \
	"voicemails", \
	"core.threads", \
	"core.uptime"]

def readline(socket):
	message = ""
	c = socket.recv(1)
	while c != "\n":
		message = message + c
		c = socket.recv(1)
	return message.strip("\r")

def query(socket, query, response = "", verbose = 0):
	if verbose:
		print query
	socket.send(query)
	message = ""
	s = readline(socket)
	while s != response:
		if verbose:
			print s
		message = message + s + "\n"
		s = readline(socket)
	return message

def login(socket, host, port, username, password, verbose = 0):
	socket.connect((host, port))
	message = query(socket, "action: login\r\nusername: " + username + \
		"\r\nsecret: " + password + \
		"\r\nevents: off\r\n\r\n", verbose = verbose)
	return message

def logout(socket, verbose = 0):
	message = query(socket, "action: logoff\r\n", verbose = verbose)
	socket.close()
	return message

def usage():
	print("usage: ./asx-stats.py [<hostname> <key>] [options]")
	print("  hostname - name of the server")
	print("  key - display value based on this key")
	print("    available keys:")
	for i in KEYS:
		print("      " + i)
	print("  options - extra options")
	print("    available options:")
	print("      -h,--help    - display this text")
	print("      -v,--verbose - verbose mode, displays AMI messages")
try:
	verbose = 0
	validargs = []
	for i in sys.argv:
		if i == "--verbose" or i == "-verbose" or i == "-v":
			verbose = 1
		elif i == "--help" or i == "-help" or i == "-h":
			usage()
			sys.exit()
		elif i == "--index" or i == "-index" or i == "-i":
			print("asterisk")
			sys.exit()
		elif i == "--get" or i == "-get" or i == "-g":
			pass
		elif i == "--query" or i == "-query" or i == "-q":
			print("asterisk:asterisk")
			sys.exit()
		else:
			validargs.append(i)
	if len(validargs) < 3:
		usage()
		sys.exit()
	config = ConfigParser.ConfigParser()
	config.read(CONFIGFILE)
	pid = os.fork()
	if pid: # parent
		socket = socket(AF_INET, SOCK_STREAM)
		login(socket, \
			config.get(validargs[1], "hostname"), \
			eval(config.get(validargs[1], "port")), \
			config.get(validargs[1], "username"), \
			config.get(validargs[1], "password"))
		result = 0
		# SIP CHANNELS
		if validargs[2] == "sip.channels":
			s = query(socket, "action: command\r\ncommand: sip show channels\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile("active SIP channel").search(i):
					result = eval(i.split()[0])
		# IAX2 CHANNELS
		elif validargs[2] == "iax2.channels":
			s = query(socket, "action: command\r\ncommand: iax2 show channels\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile("active IAX channel").search(i):
					result = eval(i.split()[0])
		# SIP PEERS
		elif validargs[2] == "sip.peers.online":
			s = query(socket, "action: command\r\ncommand: sip show peers\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile(r"sip peers \[").search(i):
					if len(i.split()) > 8:
						result = eval(i.split()[9])
					else:
						result = eval(i.split()[3][1:])
		elif validargs[2] == "sip.peers.total":
			s = query(socket, "action: command\r\ncommand: sip show peers\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile(r"sip peers \[").search(i):
					result = eval(i.split()[0])
		# IAX2 PEERS
		elif validargs[2] == "iax2.peers.online":
			s = query(socket, "action: command\r\ncommand: iax2 show peers\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile(r"iax2 peers \[").search(i):
					result = eval(i.split()[3][1:])
		elif validargs[2] == "iax2.peers.total":
			s = query(socket, "action: command\r\ncommand: iax2 show peers\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile(r"iax2 peers \[").search(i):
					result = eval(i.split()[0])
		# SIP USERS
		elif validargs[2] == "sip.users":
			s = query(socket, "action: command\r\ncommand: sip show users\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			x = 0
			for i in s:
				x = x + 1
				if re.compile("-- Remote UNIX connection").search(i):
					x = x - 1
				if re.compile("Verbosity is").search(i):
					x = x - 1
				if re.compile("Def.Context").search(i):
					x = x - 1
				if re.compile("Response:").search(i):
					x = x - 1
				if re.compile("Privilege:").search(i):
					x = x - 1
				if i == "":
					x = x - 1
			result = x
		# IAX2 USERS
		elif validargs[2] == "iax2.users":
			s = query(socket, "action: command\r\ncommand: iax2 show users\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			x = 0
			for i in s:
				x = x + 1
				if re.compile("-- Remote UNIX connection").search(i):
					x = x - 1
				if re.compile("Verbosity is").search(i):
					x = x - 1
				if re.compile("Def.Context").search(i):
					x = x - 1	
				if re.compile("Response:").search(i):
					x = x - 1
				if re.compile("Privilege:").search(i):
					x = x - 1
				if i == "":
					x = x - 1
			result = x
		# GROUP CHANNELS
		elif validargs[2] == "group.channels":
			s = query(socket, "action: command\r\ncommand: group show channels\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile("active channels").search(i):
					result = eval(i.split()[0])
		# AGENTS
		elif validargs[2] == "agents.loggedin":
			s = query(socket, "action: command\r\ncommand: agent show\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile("available at").search(i):
					result = result + 1
		elif validargs[2] == "agents.loggedout":
			s = query(socket, "action: command\r\ncommand: agent show\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile("not logged in").search(i):
					result = result + 1
		# VOICEMAILS
		elif validargs[2] == "voicemails":
			s = query(socket, "action: command\r\ncommand: voicemail show users\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile("default").search(i):
					result = result + eval(i.split()[len(i.split()) - 1])
		# CORE THREADS
		elif validargs[2] == "core.threads":
			s = query(socket, "action: command\r\ncommand: core show threads\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile("threads listed").search(i):
					result = eval(i.split()[0])
		# CORE UPTIME
		elif validargs[2] == "core.uptime":
			s = query(socket, "action: command\r\ncommand: core show uptime seconds\r\n\r\n", \
				"--END COMMAND--", verbose).split("\n")
			for i in s:
				if re.compile("System").search(i):
					result = eval(i.split()[2])
		else:
			usage()
			sys.exit()
		logout(socket, verbose)
		print result
	else: # child
		time.sleep(60)
		os.kill(pid, signal.SIGTERM)
except Exception, e:
    print e

